//=============================================================================
//
// Alazar Technologies Inc
//
// Copyright (c) 2003 Alazar Technologies Inc. All Rights Reserved.
// Unpublished - rights reserved under the Copyright laws of the
// United States And Canada.
//
// This product contains confidential information and trade secrets
// of Alazar Technologies Inc. Use, disclosure, or reproduction is
// prohibited without the prior express written permission of Alazar
// Technologies Inc
//
//=============================================================================

#include <linux/jiffies.h>
#include <linux/param.h>
#include <linux/sched.h>
#include <linux/version.h>

#include "ats_timer.h"

enum
{
    ATS_TIMER_DISABLED,
    ATS_TIMER_ENABLED,
    ATS_TIMER_AUTO
};

static void ats_timer_call(struct timer_list *list)
{
    ats_timer_t *timer = container_of(list, ats_timer_t, krn_timer);

    if (timer->callback != NULL) timer->callback(timer->context);

    if (timer->state == ATS_TIMER_AUTO) ats_timer_reset(timer);
}

rc_t ats_timer_reset(ats_timer_t *timer)
{
    if (timer == NULL) return rc_null_param;

    mod_timer(&timer->krn_timer, jiffies + msecs_to_jiffies(timer->delay));
    return rc_success;
}

rc_t ats_timer_start(ats_timer_t *timer,
                     driver_os_handle_t *driver,
                     bool restart,
                     unsigned int delay,
                     void (*callback)(void *),
                     void *context)
{
    if (timer == NULL) return rc_null_param;

    if ((timer->callback == NULL) && (callback == NULL)) return rc_null_param;

    if (callback != NULL)
    {
        timer->callback = callback;
        timer->context = context;
    }

#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 15, 0)
    init_timer(&timer->krn_timer);
    timer->krn_timer.function = (void (*)(unsigned long))ats_timer_call;
    timer->krn_timer.data = (unsigned long)&timer->krn_timer;
#else
    timer_setup(&timer->krn_timer, (void (*)(struct timer_list *))ats_timer_call, 0);
#endif
    timer->state = restart ? ATS_TIMER_AUTO : ATS_TIMER_ENABLED;
    timer->delay = delay;
    return ats_timer_reset(timer);
}

rc_t ats_timer_stop(ats_timer_t *timer)
{
    if (timer == NULL) return rc_null_param;

    timer->state = ATS_TIMER_DISABLED;
    del_timer(&timer->krn_timer);
    return rc_success;
}
