/**
 * @file
 *
 * @author Alazar Technologies Inc
 *
 * @copyright Copyright (c) 2021 Alazar Technologies Inc. All Rights
 * Reserved.  Unpublished - rights reserved under the Copyright laws
 * of the United States And Canada.
 * This product contains confidential information and trade secrets
 * of Alazar Technologies Inc. Use, disclosure, or reproduction is
 * prohibited without the prior express written permission of Alazar
 * Technologies Inc
 */

// This file is included into the dkms package.

#ifndef ATS_TIMER_H
#define ATS_TIMER_H

#include "ats_error.h"
#include "ats_os.h"

#ifdef _WIN32

#include <ntddk.h>
#include <wdf.h>

typedef struct ats_timer_s
{
    WDF_TIMER_CONFIG wdf_config;
    WDF_OBJECT_ATTRIBUTES wdf_attrs;
    WDFTIMER wdf_timer;
    unsigned int delay;
    int state;
    void (*callback)(void *);
    void *context;
} ats_timer_t;

typedef struct
{
    ats_timer_t *timer;
} ats_timer_wdf_context_t;

WDF_DECLARE_CONTEXT_TYPE_WITH_NAME(ats_timer_wdf_context_t, get_ats_timer_wdf_context)

#else

#ifdef MODULE
#include <linux/timer.h>
#endif

typedef struct ats_timer_s
{
    union
    {
#ifdef MODULE
        struct timer_list krn_timer;
#endif
        char reserved0[128];
    };
    unsigned int delay;
    int state;
    void (*callback)(void *);
    void *context;
} ats_timer_t;

#endif

/**
 *  @brief Initialize and start timer.
 *
 *  @param timer    The timer context structure pointer
 *
 *  @param driver   The driver context structure pointer
 *
 *  @param restart  The timer restart automaticaly if true
 *
 *  @param delay    The timer delay in milli-seconds
 *
 *  @param callback The function called when the timer expire
 *
 *  @param context  The context pointer used by the callback() function
 *
 */
rc_t ats_timer_start(ats_timer_t *timer,
                     driver_os_handle_t *driver,
                     bool restart,
                     unsigned int delay,
                     void (*callback)(void *),
                     void *context);

/**
 *  @brief Stop and disable timer.
 *
 *  @param timer    The timer context structure pointer
 *
 */
rc_t ats_timer_stop(ats_timer_t *timer);

/**
 *  @brief Restart timer.
 *
 *  @param timer    The timer context structure pointer
 *
 */
rc_t ats_timer_reset(ats_timer_t *timer);

#endif // ATS_TIMER_H
